# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from knack.util import CLIError
from ._utils import validate_premium_registry
from ._client_factory import cf_acr_network_rules
from typing import Optional, Union
import msrest.serialization


class VirtualNetworkRule(msrest.serialization.Model):
    """Virtual network rule.
    All required parameters must be populated in order to send to Azure.
    :ivar action: The action of virtual network rule. Possible values include: "Allow".
    :vartype action: str or ~azure.mgmt.containerregistry.v2021_08_01_preview.models.Action
    :ivar virtual_network_resource_id: Required. Resource ID of a subnet, for example:
     '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/
     providers/Microsoft.Network/virtualNetworks/{vnetName}/subnets/{subnetName}.'
    :vartype virtual_network_resource_id: str
    """

    _validation = {
        'virtual_network_resource_id': {'required': True},
    }

    _attribute_map = {
        'action': {'key': 'action', 'type': 'str'},
        'virtual_network_resource_id': {'key': 'id', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        virtual_network_resource_id: str,
        action: Optional[Union[str, "Action"]] = None,  # noqa: F821
        **kwargs
    ):
        """
        :keyword action: The action of virtual network rule. Possible values include: "Allow".
        :paramtype action: str or ~azure.mgmt.containerregistry.v2021_08_01_preview.models.Action
        :keyword virtual_network_resource_id: Required. Resource ID of a subnet, for example:
         '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/
         providers/Microsoft.Network/virtualNetworks/{vnetName}/subnets/{subnetName}.'
        :paramtype virtual_network_resource_id: str
        """
        super(VirtualNetworkRule, self).__init__(**kwargs)
        self.action = action
        self.virtual_network_resource_id = virtual_network_resource_id


NETWORK_RULE_NOT_SUPPORTED = 'Network rules are only supported for managed registries in Premium SKU.'


def acr_network_rule_list(cmd, registry_name, resource_group_name=None):
    _, resource_group_name = validate_premium_registry(
        cmd, registry_name, resource_group_name, NETWORK_RULE_NOT_SUPPORTED)

    client = cf_acr_network_rules(cmd.cli_ctx)
    registry = client.get(resource_group_name, registry_name)
    rules = registry.network_rule_set
    delattr(rules, 'default_action')
    return rules


def acr_network_rule_add(cmd,
                         client,
                         registry_name,
                         subnet=None,
                         vnet_name=None,
                         ip_address=None,
                         resource_group_name=None):
    _, resource_group_name = validate_premium_registry(
        cmd, registry_name, resource_group_name, NETWORK_RULE_NOT_SUPPORTED)

    client = cf_acr_network_rules(cmd.cli_ctx)
    registry = client.get(resource_group_name, registry_name)

    rules = registry.network_rule_set

    if subnet or vnet_name:
        rules.virtual_network_rules = rules.virtual_network_rules if rules.virtual_network_rules else []
        subnet_id = _validate_subnet(cmd.cli_ctx, subnet, vnet_name, resource_group_name)
        rules.virtual_network_rules.append(VirtualNetworkRule(virtual_network_resource_id=subnet_id))
    if ip_address:
        rules.ip_rules = rules.ip_rules if rules.ip_rules else []
        IPRule = cmd.get_models('IPRule')
        rules.ip_rules.append(IPRule(ip_address_or_range=ip_address))

    RegistryUpdateParameters = cmd.get_models('RegistryUpdateParameters')
    parameters = RegistryUpdateParameters(network_rule_set=rules)
    return client.begin_update(resource_group_name, registry_name, parameters)


def acr_network_rule_remove(cmd,
                            client,
                            registry_name,
                            subnet=None,
                            vnet_name=None,
                            ip_address=None,
                            resource_group_name=None):
    _, resource_group_name = validate_premium_registry(
        cmd, registry_name, resource_group_name, NETWORK_RULE_NOT_SUPPORTED)

    client = cf_acr_network_rules(cmd.cli_ctx)
    registry = client.get(resource_group_name, registry_name)
    rules = registry.network_rule_set

    if subnet or vnet_name:
        rules.virtual_network_rules = rules.virtual_network_rules if rules.virtual_network_rules else []
        subnet_id = _validate_subnet(cmd.cli_ctx, subnet, vnet_name, resource_group_name).lower()
        rules.virtual_network_rules = [
            x for x in rules.virtual_network_rules if x.virtual_network_resource_id.lower() != subnet_id]
    if ip_address:
        rules.ip_rules = rules.ip_rules if rules.ip_rules else []
        rules.ip_rules = [x for x in rules.ip_rules if x.ip_address_or_range != ip_address]

    RegistryUpdateParameters = cmd.get_models('RegistryUpdateParameters')
    parameters = RegistryUpdateParameters(network_rule_set=rules)
    return client.begin_update(resource_group_name, registry_name, parameters)


def _validate_subnet(cli_ctx, subnet, vnet_name, resource_group_name):
    from msrestazure.tools import is_valid_resource_id
    subnet_is_id = is_valid_resource_id(subnet)

    if subnet_is_id and not vnet_name:
        return subnet
    if subnet and not subnet_is_id and vnet_name:
        from msrestazure.tools import resource_id
        from azure.cli.core.commands.client_factory import get_subscription_id
        return resource_id(
            subscription=get_subscription_id(cli_ctx),
            resource_group=resource_group_name,
            namespace='Microsoft.Network',
            type='virtualNetworks',
            name=vnet_name,
            child_type_1='subnets',
            child_name_1=subnet)
    raise CLIError('Usage error: [--subnet ID | --subnet NAME --vnet-name NAME]')



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# pylint: disable=line-too-long,too-many-lines

from azure.cli.core.commands import LongRunningOperation
from azure.core.exceptions import HttpResponseError
from azure.mgmt.servicefabricmanagedclusters.models import (
    NodeType,
    EndpointRangeDescription,
    VMSSExtension,
    VaultSecretGroup,
    VaultCertificate,
    SubResource,
    NodeTypeActionParameters
)

from knack.log import get_logger

logger = get_logger(__name__)


# pylint:disable=too-many-locals,
def create_node_type(cmd,
                     client,
                     resource_group_name,
                     cluster_name,
                     node_type_name,
                     instance_count,
                     primary=False,
                     disk_size=None,
                     disk_type=None,
                     application_start_port=None,
                     application_end_port=None,
                     ephemeral_start_port=None,
                     ephemeral_end_port=None,
                     vm_size=None,
                     vm_image_publisher=None,
                     vm_image_offer=None,
                     vm_image_sku=None,
                     vm_image_version=None,
                     capacity=None,
                     placement_property=None,
                     is_stateless=False,
                     multiple_placement_groups=False):

    #  set defult parameters
    if disk_size is None:
        disk_size = 100

    if vm_size is None:
        vm_size = "Standard_D2"

    if vm_image_publisher is None:
        vm_image_publisher = "MicrosoftWindowsServer"

    if vm_image_offer is None:
        vm_image_offer = "WindowsServer"

    if vm_image_sku is None:
        vm_image_sku = "2019-Datacenter"

    if vm_image_version is None:
        vm_image_version = "latest"

    try:
        new_node_type = NodeType(is_primary=primary,
                                 vm_instance_count=int(instance_count),
                                 data_disk_size_gb=disk_size,
                                 data_disk_type=disk_type,
                                 vm_size=vm_size,
                                 vm_image_publisher=vm_image_publisher,
                                 vm_image_offer=vm_image_offer,
                                 vm_image_sku=vm_image_sku,
                                 vm_image_version=vm_image_version,
                                 capacities=capacity,
                                 placement_properties=placement_property,
                                 is_stateless=is_stateless,
                                 multiple_placement_groups=multiple_placement_groups)

        if application_start_port and application_end_port:
            new_node_type.application_ports = EndpointRangeDescription(start_port=application_start_port,
                                                                       end_port=application_end_port)

        if ephemeral_start_port and ephemeral_end_port:
            new_node_type.ephemeral_ports = EndpointRangeDescription(start_port=ephemeral_start_port,
                                                                     end_port=ephemeral_end_port)

        logger.info("Creating node type '%s'", node_type_name)
        poller = client.node_types.begin_create_or_update(resource_group_name, cluster_name, node_type_name, new_node_type)
        node_type = LongRunningOperation(cmd.cli_ctx)(poller)
        return node_type
    except HttpResponseError as ex:
        logger.error("HttpResponseError: %s", ex)
        raise


def update_node_type(cmd,
                     client,
                     resource_group_name,
                     cluster_name,
                     node_type_name,
                     instance_count=None,
                     application_start_port=None,
                     application_end_port=None,
                     ephemeral_start_port=None,
                     ephemeral_end_port=None,
                     capacity=None,
                     placement_property=None):
    try:
        node_type = client.node_types.get(resource_group_name, cluster_name, node_type_name)

        if instance_count is not None:
            node_type.vm_instance_count = instance_count

        if application_start_port and application_end_port:
            node_type.application_ports = EndpointRangeDescription(start_port=application_start_port,
                                                                   end_port=application_end_port)

        if ephemeral_start_port and ephemeral_end_port:
            node_type.ephemeral_ports = EndpointRangeDescription(start_port=ephemeral_start_port,
                                                                 end_port=ephemeral_end_port)

        if capacity is not None:
            node_type.capacities = capacity

        if placement_property is not None:
            node_type.placement_properties = placement_property

        poller = client.node_types.begin_create_or_update(resource_group_name, cluster_name, node_type_name, node_type)
        return LongRunningOperation(cmd.cli_ctx)(poller)
    except HttpResponseError as ex:
        logger.error("HttpResponseError: %s", ex)
        raise


def reimage_node(cmd,
                 client,
                 resource_group_name,
                 cluster_name,
                 node_type_name,
                 node_name,
                 force=False):
    try:
        nodes = [node_name] if isinstance(node_name, str) else node_name
        action_parameters = NodeTypeActionParameters(nodes=nodes, force=force)
        poller = client.node_types.begin_reimage(resource_group_name, cluster_name, node_type_name, parameters=action_parameters)
        LongRunningOperation(cmd.cli_ctx, start_msg='Reimaging nodes', finish_msg='Nodes reimaged')(poller)
    except HttpResponseError as ex:
        logger.error("HttpResponseError: %s", ex)
        raise


def restart_node(cmd,
                 client,
                 resource_group_name,
                 cluster_name,
                 node_type_name,
                 node_name,
                 force=False):
    try:
        nodes = [node_name] if isinstance(node_name, str) else node_name
        action_parameters = NodeTypeActionParameters(nodes=nodes, force=force)
        poller = client.node_types.begin_restart(resource_group_name, cluster_name, node_type_name, parameters=action_parameters)
        LongRunningOperation(cmd.cli_ctx, start_msg='Restarting nodes', finish_msg='Nodes restarted')(poller)
    except HttpResponseError as ex:
        logger.error("HttpResponseError: %s", ex)
        raise


def delete_node(cmd,
                client,
                resource_group_name,
                cluster_name,
                node_type_name,
                node_name,
                force=False):
    try:
        nodes = [node_name] if isinstance(node_name, str) else node_name
        action_parameters = NodeTypeActionParameters(nodes=nodes, force=force)
        poller = client.node_types.begin_delete_node(resource_group_name, cluster_name, node_type_name, parameters=action_parameters)
        LongRunningOperation(cmd.cli_ctx, start_msg='Deleting nodes', finish_msg='Nodes deleted')(poller)
    except HttpResponseError as ex:
        logger.error("HttpResponseError: %s", ex)
        raise


def add_vm_extension(cmd,
                     client,
                     resource_group_name,
                     cluster_name,
                     node_type_name,
                     extension_name,
                     publisher,
                     extension_type,
                     type_handler_version,
                     force_update_tag=None,
                     auto_upgrade_minor_version=True,
                     setting=None,
                     protected_setting=None,
                     provision_after_extension=None):
    try:
        node_type: NodeType = client.node_types.get(resource_group_name, cluster_name, node_type_name)

        if node_type.vm_extensions is None:
            node_type.vm_extensions = []

        newExtension = VMSSExtension(name=extension_name,
                                     publisher=publisher,
                                     type=extension_type,
                                     type_handler_version=type_handler_version,
                                     force_update_tag=force_update_tag,
                                     auto_upgrade_minor_version=auto_upgrade_minor_version,
                                     settings=setting,
                                     protected_settings=protected_setting,
                                     provision_after_extensions=provision_after_extension)

        node_type.vm_extensions.append(newExtension)

        poller = client.node_types.begin_create_or_update(resource_group_name, cluster_name, node_type_name, node_type)
        return LongRunningOperation(cmd.cli_ctx)(poller)
    except HttpResponseError as ex:
        logger.error("HttpResponseError: %s", ex)
        raise


def delete_vm_extension(cmd,
                        client,
                        resource_group_name,
                        cluster_name,
                        node_type_name,
                        extension_name):
    try:
        node_type: NodeType = client.node_types.get(resource_group_name, cluster_name, node_type_name)

        if node_type.vm_extensions is not None:
            original_len = len(node_type.vm_extensions)
            node_type.vm_extensions = list(filter(lambda x: x.name.lower() != extension_name.lower(), node_type.vm_extensions))
            if original_len == len(node_type.vm_extensions):
                raise 'Extension with name {} not found'.format(extension_name)

        poller = client.node_types.begin_create_or_update(resource_group_name, cluster_name, node_type_name, node_type)
        return LongRunningOperation(cmd.cli_ctx)(poller)
    except HttpResponseError as ex:
        logger.error("HttpResponseError: %s", ex)
        raise


def add_vm_secret(cmd,
                  client,
                  resource_group_name,
                  cluster_name,
                  node_type_name,
                  source_vault_id,
                  certificate_url,
                  certificate_store):
    try:
        node_type: NodeType = client.node_types.get(resource_group_name, cluster_name, node_type_name)

        if node_type.vm_secrets is None:
            node_type.vm_secrets = []

        vault = next((x for x in node_type.vm_secrets if x.source_vault.id.lower() == source_vault_id.lower()), None)

        vault_certificate = VaultCertificate(certificate_store=certificate_store, certificate_url=certificate_url)
        new_vault_secret_group = False
        if vault is None:
            new_vault_secret_group = True
            source_vault = SubResource(id=source_vault_id)
            vault = VaultSecretGroup(source_vault=source_vault, vault_certificates=[])

        if vault.vault_certificates is None:
            vault.vault_certificates = []

        vault.vault_certificates.append(vault_certificate)

        if new_vault_secret_group:
            node_type.vm_secrets.append(vault)

        poller = client.node_types.begin_create_or_update(resource_group_name, cluster_name, node_type_name, node_type)
        return LongRunningOperation(cmd.cli_ctx)(poller)
    except HttpResponseError as ex:
        logger.error("HttpResponseError: %s", ex)
        raise



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------


def get_botservice_management_client(cli_ctx, *_):
    from azure.cli.core.commands.client_factory import get_mgmt_service_client
    from azure.mgmt.botservice import AzureBotService
    return get_mgmt_service_client(cli_ctx, AzureBotService)


def get_botOperations_client(cli_ctx, *_):
    return get_botservice_management_client(cli_ctx).bots


def get_botChannels_client(cli_ctx, *_):
    return get_botservice_management_client(cli_ctx).channels


def get_operations_client(cli_ctx, *_):
    return get_botservice_management_client(cli_ctx).operations


def get_botConnections_client(cli_ctx, *_):
    return get_botservice_management_client(cli_ctx).bot_connection



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


@register_command_group(
    "monitor log-analytics workspace linked-service",
)
class __CMDGroup(AAZCommandGroup):
    """Manage linked service for log analytics workspace.

    Linked services is used to define a relation from the workspace to another Azure resource. Log Analytics and Azure resources then leverage this connection in their operations. Example uses of Linked Services in Log Analytics workspace are Automation account and workspace association to CMK.
    """
    pass


__all__ = ["__CMDGroup"]
